import numpy as np
import math
import matplotlib.pyplot as plt

def f(x):
    r2 = 0.
    for i in range(dim):
        r2 = r2 + x[i,:]**2
        fx = 1.*(r2 <= 1)
    return fx

def mc_experiments(func,a,b,N,M,dim):
    # Fuehre Experimente durch
    I = np.zeros(M)
    for i in np.arange(M):
        [I[i],ivar] = mcquad(func,a,b,N,dim)

    # Mittelwert und Varianz
    Imean = np.mean(I)
    Ivar = np.var(I)
    
    return Imean,Ivar

def mcquad(func,a,b,N,dim):
    
    # Erzeuge N samples in [a,b]
    x = np.random.rand(dim,N)
    for i in range(dim):
        x[i,:] = a[i] + (b[i] - a[i])*x[i,:]
    
    # Volumen von [a,b]
    vol = 1.
    for i in range(dim):
        vol = vol*(b[i] - a[i])
        
    # Werte Funktion an Samplepunkten aus
    fx = func(x)
    
    I = np.mean(fx)
    var2 = np.var(fx)
    
    return I*vol, var2*vol**2

# Dimension
dim = 4
a = -1.0*np.ones(dim)
b = 1.0*np.ones(dim)
# Analytische Loesung
Ianalytical = np.pi**(dim/2.)/math.gamma(dim/2.+1) 
# Anzahl Experimente
M = 1
# Anzahl Samples
N = 10**np.arange(7)

Imean_arr = []
Ivar_arr = []
error_arr = []

for Nk in N:
    [Imean_k,Ivar_k] = mc_experiments(f,a,b,Nk,M,dim)
    Imean_arr.append(Imean_k)
    Ivar_arr.append(Ivar_k)
    error_arr.append(abs(Imean_k - Ianalytical))

plt.figure()
plt.title(r'Monte-Carlo-Integration', fontsize=26.)
plt.plot(N, error_arr, 'o', label=r'Monte-Carlo')
plt.plot(N, 10*N**-0.5, label=r'$\mathcal{O}(N^{-1/2})$')
plt.xlabel(r'$N$')
plt.ylabel(r'Error')
plt.xscale('log')
plt.yscale('log')
plt.legend(loc=1)
plt.tight_layout()
plt.show()